@@ -1,64 +1,68 @@
-# -*- coding: utf-8 -*-
 
-import os
-import pathlib
+# Imports from standard library
+from pathlib import Path
 import sys
 from typing import Optional
-from PyQt5.QtCore import QSettings, QDataStream, QIODevice, QUrl, QByteArray
-from PyQt5.QtXml  import QDomDocument
 
+# third party imports
+from qtpy import QT5
+from qtpy.QtCore import (
+    QSettings, QDataStream, QIODevice, QUrl, QByteArray)
+if not QT5:
+    from qtpy.QtCore import QIODeviceBase
+
+from qtpy.QtXml  import QDomDocument, QDomNode
+
+# Imports from src/shared
 import ray
-from daemon_tools import get_app_config_path, dirname
+
+# Local imports
+from daemon_tools import get_app_config_path
 
 QFILEDIALOG_MAGIC = 190
 
 
 class PickerType:
-    def __init__(self, config_path):
-        self.config_path = config_path
+    def __init__(self, config_path: Path):
+        self._config_path = config_path
         self.written = False
 
-    def _get_contents(self):
-        if os.path.exists(self.config_path):
+    def _get_contents(self) -> Optional[str]:
+        if self._config_path.exists():
             try:
-                file = open(self.config_path, 'r')
-                contents = file.read()
-                file.close()
+                with open(self._config_path, 'r') as file:
+                    contents = file.read()                    
                 return contents
             except:
                 return None
         else:
             return ""
 
-    def _print_contents(self, contents):
+    def _print_contents(self, contents: str) -> bool:
         try:
-            file = open(self.config_path, 'w')
+            with open(self._config_path, 'w') as file:
+                file.write(contents)
         except:
             return False
-
-        file.write(contents)
-        file.close()
-
         return True
 
-    def make_bookmark(self, spath):
+    def make_bookmark(self, spath: Path):
         pass
 
-    def remove_bookmark(self, spath):
+    def remove_bookmark(self, spath: Path):
         pass
 
 class PickerTypeGtk(PickerType):
-    def make_bookmark(self, spath):
+    def make_bookmark(self, spath: Path):
         if self.written:
             return
 
-        url = pathlib.Path(spath).as_uri()
+        url = spath.as_uri()
+        config_dir = self._config_path.parent
 
-        config_dir = dirname(self.config_path)
-
-        if not os.path.exists(config_dir):
+        if not config_dir.exists():
             try:
-                os.makedirs(config_dir)
+                config_dir.mkdir(parents=True)
             except:
                 return
 
@@ -76,11 +80,11 @@ class PickerTypeGtk(PickerType):
         if self._print_contents(contents):
             self.written = True
 
-    def remove_bookmark(self, spath):
+    def remove_bookmark(self, spath: Path):
         if not self.written:
             return
 
-        url = pathlib.Path(spath).as_uri()
+        url = spath.as_uri()
 
         contents = self._get_contents()
         if not contents:
@@ -104,7 +108,7 @@ class PickerTypeGtk(PickerType):
         self.written = False
 
 class PickerTypeFltk(PickerType):
-    def make_bookmark(self, spath):
+    def make_bookmark(self, spath: Path):
         if self.written:
             return
 
@@ -124,7 +128,7 @@ class PickerTypeFltk(PickerType):
                 fav0_found = True
 
                 if line.partition(':')[2]:
-                    if line.partition(':')[2] == spath:
+                    if line.partition(':')[2] == str(spath):
                         #bookmark already written, do nothing
                         return
 
@@ -133,7 +137,7 @@ class PickerTypeFltk(PickerType):
                         num = int(num_s)
                 else:
                     if not empty_fav:
-                        line += spath
+                        line += str(spath)
                         empty_fav = True
 
             if line or not fav0_found:
@@ -146,7 +150,7 @@ class PickerTypeFltk(PickerType):
         if self._print_contents(contents):
             self.written = True
 
-    def remove_bookmark(self, spath):
+    def remove_bookmark(self, spath: Path):
         if not self.written:
             return
 
@@ -156,18 +160,18 @@ class PickerTypeFltk(PickerType):
             return
 
         lines = contents.split('\n')
-        favorites = []
+        favorites = list[str]()
 
         for line in lines:
             if line.startswith('favorite') and ':' in line:
                 fav = line.partition(':')[2]
                 favorites.append(fav)
 
-        if not spath in favorites:
+        if not str(spath) in favorites:
             self.written = False
             return
 
-        favorites.remove(spath)
+        favorites.remove(str(spath))
         contents = ""
         num = 0
 
@@ -186,22 +190,26 @@ class PickerTypeFltk(PickerType):
         self.written = False
 
 class PickerTypeQt4(PickerType):
-    def make_bookmark(self, spath):
+    def make_bookmark(self, spath: Path):
         if self.written:
             return
 
-        if not os.path.exists(self.config_path):
+        if not self._config_path.exists():
             #do not write shortcuts if file was not created by Qt4 himself
             return
 
-        url = pathlib.Path(spath).as_uri()
+        url = spath.as_uri()
 
-        settings_qt4 = QSettings(self.config_path, QSettings.IniFormat)
+        settings_qt4 = QSettings(str(self._config_path),
+                                 QSettings.Format.IniFormat)
         if not settings_qt4.isWritable():
             return
 
         data = settings_qt4.value('Qt/filedialog')
-        stream = QDataStream(data, QIODevice.ReadOnly)
+        if QT5:
+            stream = QDataStream(data, QIODevice.ReadOnly)
+        else:
+            stream = QDataStream(data, QIODeviceBase.OpenModeFlag.ReadOnly)
 
         magic = stream.readUInt32()
         version = stream.readUInt32()
@@ -216,7 +224,7 @@ class PickerTypeQt4(PickerType):
             qUrl = QUrl()
             stream >> qUrl
 
-            if qUrl.isLocalFile() and qUrl.toLocalFile() == spath:
+            if qUrl.isLocalFile() and qUrl.toLocalFile() == str(spath):
                 #spath already in qt4 bookmarks
                 return
 
@@ -231,7 +239,12 @@ class PickerTypeQt4(PickerType):
         #now rewrite bytes
 
         new_data = QByteArray()
-        new_stream = QDataStream(new_data, QIODevice.WriteOnly)
+        
+        if QT5:
+            new_stream = QDataStream(new_data, QIODevice.WriteOnly)
+        else:
+            new_stream = QDataStream(
+                new_data, QIODeviceBase.OpenModeFlag.WriteOnly)
 
         new_stream.writeUInt32(magic)
         new_stream.writeUInt32(3)
@@ -253,23 +266,27 @@ class PickerTypeQt4(PickerType):
 
         self.written = True
 
-    def remove_bookmark(self, spath):
+    def remove_bookmark(self, spath: Path):
         if not self.written:
             return
 
-        if not os.path.exists(self.config_path):
+        if not self._config_path.exists():
             self.written = False
             return
 
-        url = pathlib.Path(spath).as_uri()
+        url = spath.as_uri()
 
-        settings_qt4 = QSettings(self.config_path, QSettings.IniFormat)
+        settings_qt4 = QSettings(
+            str(self._config_path), QSettings.Format.IniFormat)
         if not settings_qt4.isWritable():
             self.written = False
             return
 
         data = settings_qt4.value('Qt/filedialog')
-        stream = QDataStream(data, QIODevice.ReadOnly)
+        if QT5:
+            stream = QDataStream(data, QIODevice.ReadOnly)
+        else:
+            stream = QDataStream(data, QIODeviceBase.OpenModeFlag.ReadOnly)
 
         magic = stream.readUInt32()
         version = stream.readUInt32()
@@ -286,7 +303,7 @@ class PickerTypeQt4(PickerType):
             qUrl = QUrl()
             stream >> qUrl
 
-            if qUrl.isLocalFile() and qUrl.toLocalFile() == spath:
+            if qUrl.isLocalFile() and qUrl.toLocalFile() == str(spath):
                 bookmark_found = True
             else:
                 bookmarks.append(qUrl)
@@ -308,7 +325,12 @@ class PickerTypeQt4(PickerType):
         #now rewrite bytes
 
         new_data = QByteArray()
-        new_stream = QDataStream(new_data, QIODevice.WriteOnly)
+        
+        if QT5:
+            new_stream = QDataStream(new_data, QIODevice.WriteOnly)
+        else:
+            new_stream = QDataStream(
+                new_data, QIODeviceBase.OpenModeFlag.WriteOnly)
 
         new_stream.writeUInt32(magic)
         new_stream.writeUInt32(3)
@@ -331,17 +353,18 @@ class PickerTypeQt4(PickerType):
         self.written = False
 
 class PickerTypeQt5(PickerType):
-    def make_bookmark(self, spath):
+    def make_bookmark(self, spath: Path):
         if self.written:
             return
 
-        if not os.path.exists(self.config_path):
+        if not self._config_path.exists():
             #do not write shortcuts if file was not created by Qt5 himself
             return
 
-        url = pathlib.Path(spath).as_uri()
+        url = spath.as_uri()
 
-        settings_qt5 = QSettings(self.config_path, QSettings.IniFormat)
+        settings_qt5 = QSettings(
+            str(self._config_path), QSettings.Format.IniFormat)
         if not settings_qt5.isWritable():
             return
 
@@ -350,7 +373,7 @@ class PickerTypeQt5(PickerType):
 
         for sc in shortcuts:
             sc_url = QUrl(sc)
-            if sc_url.isLocalFile() and sc_url.toLocalFile() == spath:
+            if sc_url.isLocalFile() and sc_url.toLocalFile() == str(spath):
                 return
 
         shortcuts.append(url)
@@ -359,21 +382,22 @@ class PickerTypeQt5(PickerType):
         settings_qt5.sync()
         self.written = True
 
-    def remove_bookmark(self, spath):
+    def remove_bookmark(self, spath: Path):
         if not self.written:
             return
 
-        if not os.path.exists(self.config_path):
+        if not self._config_path.exists():
             self.written = False
             return
 
-        settings_qt5 = QSettings(self.config_path, QSettings.IniFormat)
-        shortcuts = ray.get_list_in_settings(settings_qt5,
-                                             'FileDialog/shortcuts')
+        settings_qt5 = QSettings(
+            str(self._config_path), QSettings.Format.IniFormat)
+        shortcuts = ray.get_list_in_settings(
+            settings_qt5, 'FileDialog/shortcuts')
 
         for sc in shortcuts:
             sc_url = QUrl(sc)
-            if sc_url.isLocalFile() and sc_url.toLocalFile() == spath:
+            if sc_url.isLocalFile() and sc_url.toLocalFile() == str(spath):
                 shortcuts.remove(sc)
                 break
         else:
@@ -385,7 +409,7 @@ class PickerTypeQt5(PickerType):
         self.written = False
 
 class PickerTypeKde5(PickerType):
-    def make_bookmark(self, spath):
+    def make_bookmark(self, spath: Path):
         if self.written:
             return
 
@@ -394,7 +418,7 @@ class PickerTypeKde5(PickerType):
             # we won't write a file for kde5 if file doesn't already exists
             return
 
-        url = pathlib.Path(spath).as_uri()
+        url = spath.as_uri()
 
         xml = QDomDocument()
         xml.setContent(contents)
@@ -415,7 +439,7 @@ class PickerTypeKde5(PickerType):
         bk = xml.createElement('bookmark')
         bk.setAttribute('href', url)
         title = xml.createElement('title')
-        title_text = xml.createTextNode(os.path.basename(spath))
+        title_text = xml.createTextNode(spath.name)
         title.appendChild(title_text)
         bk.appendChild(title)
         content.appendChild(bk)
@@ -423,8 +447,7 @@ class PickerTypeKde5(PickerType):
         if self._print_contents(xml.toString()):
             self.written = True
 
-
-    def remove_bookmark(self, spath):
+    def remove_bookmark(self, spath: Path):
         if not self.written:
             return
 
@@ -433,7 +456,7 @@ class PickerTypeKde5(PickerType):
             self.written = False
             return
 
-        url = pathlib.Path(spath).as_uri()
+        url = spath.as_uri()
 
         xml = QDomDocument()
         xml.setContent(contents)
@@ -461,31 +484,31 @@ class PickerTypeKde5(PickerType):
 
 class BookMarker:
     def __init__(self):
-        self._bookmarks_memory = "%s/bookmarks.xml" % get_app_config_path()
+        self._bookmarks_memory = get_app_config_path() / 'bookmarks.xml'
         self._daemon_port = 0
 
-        home = os.getenv('HOME')
+        home = Path.home()
 
-        self._gtk2 = PickerTypeGtk("%s/.gtk-bookmarks" % home)
-        self._gtk3 = PickerTypeGtk("%s/.config/gtk-3.0/bookmarks" % home)
+        self._gtk2 = PickerTypeGtk(home / '.gtk-bookmarks')
+        self._gtk3 = PickerTypeGtk(home / '.config/gtk-3.0/bookmarks')
         self._fltk = PickerTypeFltk(
-            "%s/._fltk/fltk.org/filechooser.prefs" % home)
-        self._kde5 = PickerTypeKde5("%s/.local/share/user-places.xbel" % home)
-        self._qt4 = PickerTypeQt4("%s/.config/Trolltech.conf" % home)
-        self._qt5 = PickerTypeQt5("%s/.config/QtProject.conf" % home)
+            home / '._fltk/fltk.org/filechooser.prefs')
+        self._kde5 = PickerTypeKde5(home / '.local/share/user-places.xbel')
+        self._qt4 = PickerTypeQt4(home / '.config/Trolltech.conf')
+        self._qt5 = PickerTypeQt5(home / '.config/QtProject.conf')
 
     def _get_xml(self) -> Optional[QDomDocument]:
         xml = QDomDocument()
         file_exists = False
 
-        if os.path.exists(self._bookmarks_memory):
+        if self._bookmarks_memory.exists():
             try:
-                file = open(self._bookmarks_memory, 'r')
-                xml.setContent(file.read())
+                with open(self._bookmarks_memory, 'r') as file:
+                    xml.setContent(file.read())
                 file_exists = True
             except:
                 try:
-                    os.remove(self._bookmarks_memory)
+                    self._bookmarks_memory.unlink()
                 except:
                     return None
 
@@ -497,12 +520,12 @@ class BookMarker:
 
     def _write_xml_file(self, xml: QDomDocument):
         try:
-            file = open(self._bookmarks_memory, 'w')
-            file.write(xml.toString())
+            with open(self._bookmarks_memory, 'w') as file:
+                file.write(xml.toString())
         except:
             return
 
-    def _get_pickers_for_xml(self):
+    def _get_pickers_for_xml(self) -> str:
         string = ":"
         if self._gtk2.written:
             string += "gtk2:"
@@ -522,7 +545,7 @@ class BookMarker:
     def set_daemon_port(self, port):
         self._daemon_port = port
 
-    def make_all(self, spath):
+    def make_all(self, spath: Path):
         for picker in (self._gtk2, self._gtk3, self._fltk,
                        self._kde5, self._qt4, self._qt5):
             picker.make_bookmark(spath)
@@ -535,14 +558,14 @@ class BookMarker:
 
         bke = xml.createElement('bookmarker')
         bke.setAttribute('port', self._daemon_port)
-        bke.setAttribute('session_path', spath)
+        bke.setAttribute('session_path', str(spath))
         bke.setAttribute('pickers', self._get_pickers_for_xml())
 
         xml_content.appendChild(bke)
 
         self._write_xml_file(xml)
 
-    def remove_all(self, spath):
+    def remove_all(self, spath: Path):
         for picker in (self._gtk2, self._gtk3, self._fltk,
                        self._kde5, self._qt4, self._qt5):
             picker.remove_bookmark(spath)
@@ -562,21 +585,21 @@ class BookMarker:
 
             if (port.isdigit()
                     and int(port) == self._daemon_port
-                    and session_path == spath):
+                    and session_path == str(spath)):
                 xml_content.removeChild(node)
                 break
 
         self._write_xml_file(xml)
 
 
-    def clean(self, all_session_paths):
+    def clean(self, all_session_paths: list[str]):
         xml = self._get_xml()
         if not xml:
             return
 
         xml_content = xml.documentElement()
         nodes = xml_content.childNodes()
-        nodes_to_remove = []
+        nodes_to_remove = list[QDomNode]()
 
         for i in range(nodes.count()):
             node = nodes.at(i)
@@ -588,25 +611,27 @@ class BookMarker:
                 nodes_to_remove.append(node)
                 continue
 
+            pspath = Path(spath)
+
             if not spath in all_session_paths:
                 if ":gtk2:" in pickers:
                     self._gtk2.written = True
-                    self._gtk2.remove_bookmark(spath)
+                    self._gtk2.remove_bookmark(pspath)
                 if ":gtk3:" in pickers:
                     self._gtk3.written = True
-                    self._gtk3.remove_bookmark(spath)
+                    self._gtk3.remove_bookmark(pspath)
                 if ":fltk:" in pickers:
                     self._fltk.written = True
-                    self._fltk.remove_bookmark(spath)
+                    self._fltk.remove_bookmark(pspath)
                 if ":kde5:" in pickers:
                     self._kde5.written = True
-                    self._kde5.remove_bookmark(spath)
+                    self._kde5.remove_bookmark(pspath)
                 if ":qt4:" in pickers:
                     self._qt4.written = True
-                    self._qt4.remove_bookmark(spath)
+                    self._qt4.remove_bookmark(pspath)
                 if ":qt5:" in pickers:
                     self._qt5.written = True
-                    self._qt5.remove_bookmark(spath)
+                    self._qt5.remove_bookmark(pspath)
 
                 nodes_to_remove.append(node)
 
@@ -615,6 +640,7 @@ class BookMarker:
 
         self._write_xml_file(xml)
 
+
 if __name__ == '__main__':
     bm_maker = BookMarker()
-    bm_maker.make_all(sys.argv[1])
+    bm_maker.make_all(Path(sys.argv[1]))
